﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;
using Microsoft.VisualStudio.QualityTools.NetworkEmulation;
using Path = System.IO.Path;
using System.ComponentModel;

namespace NetworkEmulator
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private NetworkEmulationDriver driver = null;

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            string profilesDirPath = Path.Combine(
                Environment.CurrentDirectory, 
                "Profiles");
            DirectoryInfo profilesDir = new DirectoryInfo(profilesDirPath);
            var profiles = from c in profilesDir.GetFiles("*.network")
                           select Path.GetFileNameWithoutExtension(c.Name);
            NetworkProfilesList.ItemsSource = profiles;
            driver = new NetworkEmulationDriver();

            try
            {
                if (!driver.Initialize())
                {
                    MessageBox.Show("There was an error initializing the network emulator.");
                }
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }

        }

        private void Window_Closing(object sender, CancelEventArgs e)
        {
            string stopMessage = null;

            if (StopButton.IsEnabled)
            {
                bool isStopSuccess = false;
            
                for (int i = 0; i < 3 && !isStopSuccess; ++i)
                {
                    this.Title = String.Format(
                        "Attempting to stop network emulation, attempt {0} of 3...", 
                        i + 1);
                    try
                    {
                        isStopSuccess = driver.StopEmulation(2000, true);
                    }
                    catch (Exception ex)
                    {
                        HandleException(ex);
                        break;
                    }
                }

                if(isStopSuccess)
                {
                    stopMessage = "Emulation stopped successfully.";
                }
                else
                {
                    stopMessage = "Failed to stop emulation.";
                }
            }

            this.Title = "Cleaning up network emulation driver...";
            string cleanupMessage = null;

            bool isCleanupSuccess = false;

            try
            {
                isCleanupSuccess = driver.Cleanup();
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }

            if (isCleanupSuccess)
            {
                cleanupMessage = "Emulation was cleaned up successfully.";
            }
            else
            {
                cleanupMessage = "Failed to clean up emulation.";
            }

            if (!String.IsNullOrEmpty(stopMessage))
            {
                stopMessage += "\r\n";
            }

            this.Title = "Closing...";
            MessageBox.Show(stopMessage + cleanupMessage, "Cleanup Summary");
        }

        private void StartButton_Click(object sender, RoutedEventArgs e)
        {
            string profileFileName = (String)NetworkProfilesList.SelectedItem + ".network";
            string profilePath = Path.Combine(
                Environment.CurrentDirectory, 
                "Profiles", 
                profileFileName);

            bool isLoadSuccess = false;

            try
            {
                isLoadSuccess = driver.LoadProfile(profilePath);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return;
            }

            if (!isLoadSuccess)
            {
                MessageBox.Show("There was an error loading profile.");
                return;
            }

            bool isStartSuccess = false;

            try
            {
                isStartSuccess = driver.StartEmulation();
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return;
            }

            if (isStartSuccess)
            {
                StartButton.IsEnabled = false;
                NetworkProfilesList.IsEnabled = false;
                StopButton.IsEnabled = true;
            }
            else
            {
                MessageBox.Show("There was an error starting the emulation.");
            }
        }

        private void StopButton_Click(object sender, RoutedEventArgs e)
        {
            bool isStopSuccess = false;

            try
            {
                isStopSuccess = driver.StopEmulation(2000, true);
            }
            catch (Exception ex)
            {
                HandleException(ex);
                return;
            }

            if (isStopSuccess)
            {
                StartButton.IsEnabled = true;
                StopButton.IsEnabled = false;
                NetworkProfilesList.IsEnabled = true;
            }
            else
            {
                MessageBox.Show("There was an error stopping the emulation.");
            }
        }

        private void NetworkProfilesList_SelectionChanged(
            object sender, 
            SelectionChangedEventArgs e)
        {
            if (-1 == NetworkProfilesList.SelectedIndex)
            {
                StartButton.IsEnabled = false;
                CurrentProfileLabel.Content = null;
            }
            else
            {
                StartButton.IsEnabled = true;
                CurrentProfileLabel.Content = NetworkProfilesList.SelectedItem;
            }
        }

        private void HandleException(Exception ex)
        {
            StartButton.IsEnabled = false;
            NetworkProfilesList.IsEnabled = false;
            StopButton.IsEnabled = false;

            MessageBox.Show(
                String.Format(
                    "{0} was thrown by network emulation driver.\r\nMessage:{1}",
                    ex.GetType(),
                    ex.Message),
                "Unhandled Exception",
                MessageBoxButton.OK,
                MessageBoxImage.Error);
        }
    }
}
